/**
 * XmlSerializer Workshop Copyright (c) Simon Steele, 2003
 * see included license.txt for licensing.
 */
using System;
using System.Collections;
using System.IO;
using System.Windows.Forms;
using System.Runtime.InteropServices;

using XmlSerializerWorkshop.DomainGlue;

namespace XmlSerializerWorkshop.Shell
{
	/// <summary>Class to delete files using SHFileOperation.</summary>
	public class ShellDeleter
	{
		string deleteSet = "";

		/// <summary>Add a file to the set of files to be deleted.</summary>
		/// <param name="fn">Fully-qualified filename.</param>
		public void AddFile(string fn)
		{
			FileInfo fi = new FileInfo(fn);
			if(fi.Exists)
			{
				deleteSet += fn;
				deleteSet += "\0";
			}
		}

		/// <summary>Call out to SHFileOperation to delete the list of files.</summary>
		/// <returns>True on successful deletion.</returns>
		public bool Delete()
		{
			if(deleteSet == null || deleteSet.Length == 0)
				return false;

			deleteSet += "\0";

			SHFILEOPSTRUCT fileOp = new SHFILEOPSTRUCT();
			fileOp.hwnd = IntPtr.Zero;
			fileOp.wFunc = /*FO_DELETE*/3;
			fileOp.pFrom = Marshal.StringToHGlobalUni(deleteSet);
			fileOp.pTo = IntPtr.Zero;
			fileOp.fFlags = /*FOF_NOCONFIRMATION*/0x0010;

			if(SHFileOperation(ref fileOp) != 0)
				return false;

			return true;
		}

		[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Auto, Pack=1)]
		struct SHFILEOPSTRUCT
		{
			public IntPtr hwnd;
			[MarshalAs(UnmanagedType.U4)] public int wFunc;
			public IntPtr pFrom;
			public IntPtr pTo;
			public short fFlags;
			[MarshalAs(UnmanagedType.Bool)] public bool fAnyOperationsAborted;
			public IntPtr hNameMappings;
			public string lpszProgressTitle;
		}

		[DllImport("shell32.dll", CharSet=CharSet.Auto)]
		static extern int SHFileOperation(ref SHFILEOPSTRUCT FileOp);
	}

	/// <summary>Class storing the details of a temporary assembly to be deleted.</summary>
	public class ToDelete
	{
		string name_;
		string path_;

		/// <summary>Base name of the assembly.</summary>
		public string Name
		{
			get { return name_; }
		}

		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="name">Base name of the XmlSerializer created assembly.</param>
		/// <param name="path">Path to the temporary files.</param>
		public ToDelete(string name, string path)
		{
			name_ = name;
			path_ = path;
		}

		/// <summary>
		/// Try to delete all of the files that the XmlSerializer 
		/// code left in the temp directory.
		/// </summary>
		/// <param name="bNuke">True to delete rather than send to the recycling bin.</param>
		/// <returns>True on success.</returns>
		public bool DeleteFiles(bool bNuke)
		{
			ShellDeleter deleter = new ShellDeleter();

			string baseFile = Path.Combine(path_, name_);
			bool bHaveFiles = false;

			string[] extarray = new string[]{".0.cs", ".cmdline", ".err", ".out", ".pdb", ".dll"};
			foreach(string ext in extarray)
			{
				string actualFile = Path.ChangeExtension(baseFile, ext);
				System.IO.FileInfo fi = new System.IO.FileInfo(actualFile);
				if(fi.Exists)
				{
					bHaveFiles = true;
					
					if(!bNuke)
					{
						deleter.AddFile(actualFile);
					}
					else
					{
						fi.Delete();
					}
				}
			}

			if(bHaveFiles)
			{
				if(!bNuke)
					deleter.Delete();
				
				return true;
			}
			else
				return false;
		}
	}

	/// <summary>Outer shell that creates an <see cref="AppDomain"/>to 
	/// run the workshop in.</summary>
	[Serializable]
	public class Shell : MarshalByRefObject
	{
		ArrayList tempfiles_ = new ArrayList();

		/// <summary>
		/// Run the workshop.
		/// </summary>
		public void Run()
		{
			AppDomain adRunner = AppDomain.CreateDomain("XmlSerializerWorkshopRunnerDomain");
			System.Reflection.Assembly wsLib = adRunner.Load("WorkshopLib");
			System.Runtime.Remoting.ObjectHandle runnerObj = null;
			IRunner runner = null;
			
			if(wsLib != null)
			{
				// Create an instance of WorkshopLib.Runner in the new app domain.
				runnerObj = adRunner.CreateInstance("WorkshopLib", "WorkshopLib.Runner");
				if(runnerObj != null)
					runner = (IRunner)runnerObj.Unwrap();
			}

			if(runner != null)
			{
				runner.OnAssemblyCreated += new AssemblyCreatedDelegate(runner_OnAssemblyCreated);
				runner.Run();

				AppDomain.Unload(adRunner);
				
				System.Reflection.Assembly [] As = AppDomain.CurrentDomain.GetAssemblies();
				foreach(System.Reflection.Assembly a in As)
				{
					System.Diagnostics.Debug.WriteLine(a.GetName().Name);
				}

				bool bDeleteTempFiles = true;

				string sTemp = System.Configuration.ConfigurationSettings.AppSettings["DeleteTempFiles"];
				if(sTemp != null && sTemp != "")
					bDeleteTempFiles = Boolean.Parse(sTemp);

				if(bDeleteTempFiles)
					deleteFiles();
			}
		}

		/// <summary>
		/// Delete temporary files created by the workshop.
		/// </summary>
		void deleteFiles()
		{
			string sTemp = System.Configuration.ConfigurationSettings.AppSettings["NukeTempFiles"];

			bool bNukeDelete = false;

			if(sTemp != null && sTemp != "")
				bNukeDelete = Boolean.Parse(sTemp);

			foreach(ToDelete td in tempfiles_)
			{
				if(!td.DeleteFiles(bNukeDelete))
					System.Diagnostics.Debug.WriteLine("Could not delete " + td.Name);
			}
			tempfiles_.Clear();
		}

		/// <summary>
		/// Called through remoting from the other AppDomain to add a temporary
		/// assembly to the list of those to be deleted.
		/// </summary>
		/// <param name="basename">Basename of the assembly.</param>
		/// <param name="location">Location of the temporary files.</param>
		public void runner_OnAssemblyCreated(string basename, string location)
		{
			tempfiles_.Add(new ToDelete(basename, location));
		}

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{			
			Shell shell = new Shell();
			shell.Run();
		}
	}
}
